@@ -5,10 +5,11 @@ from aquests.protocols.grpc import discover
 from sqlphile import Template, DB_PGSQL
 from skitai.wsgi_apps import Config
 import skitai
+import sys
 from .app import mixin
 
 class Atila (mixin.MixIn):
-    PRESERVES_ON_RELOAD = ["reloadables"]
+    PRESERVES_ON_RELOAD = ["reloadables", "service_roots"]
     
     def __init__ (self, app_name):
         mixin.MixIn.__init__ (self)
@@ -32,7 +33,7 @@ class Atila (mixin.MixIn):
         
         class Client (client.Client):        
             def make_request (self, *args, **karg):
-                request = client.Client.make_request (self, *args, **karg)
+                request = client.Client.make_request (self, *args, **karg)                
                 return self.handle_request (request)
                 
             def __enter__ (self):
@@ -57,18 +58,18 @@ class Atila (mixin.MixIn):
             except AttributeError:     
                 self.authenticate = "digest"
         
-        # temlates --------------------------------------------
         self.setup_template_engines (path)
-            
-        # unbinded sqlphile -----------------------------------
         self.initialize_sqlphile (path)
         
         # vaild packages --------------------------------------
-        for d in self.PACKAGE_DIRS:
+        for d in self.PACKAGE_DIRS:            
             maybe_dir = os.path.join (path, d)
             if os.path.isdir (maybe_dir):
                 self.add_package_dir (maybe_dir)
-                
+            if d in sys.modules:
+                # print ('~~~~~~', d, sys.modules [d])
+                self.service_roots.append (sys.modules [d])                
+
         if module:
             self.find_mountables (module)
         self.mount_externals () # these're not auto reloadad
