@@ -15,7 +15,8 @@ class Decorators:
         }
         self._reloading = False
         self._function_specs = {}
-        self._current_function_specs = {}        
+        self._current_function_specs = {}  
+        self._parameter_caches = {}
         self._conditions = {}        
         self._cond_check_lock = threading.RLock ()                
         self._binds_server = [None] * 7
@@ -31,7 +32,7 @@ class Decorators:
         func_id = self.get_func_id (func)
         if func_id not in self._function_specs or func_id not in self._current_function_specs:
             # save origin spec
-            self._function_specs [func_id] = inspect.getfullargspec(func)
+            self._function_specs [func_id] = inspect.getfullargspec (func)
             self._current_function_specs [func_id] = None
     
     def get_function_spec (self, func):
@@ -92,7 +93,8 @@ class Decorators:
     
     def testpass_required (self, testfunc):
         def decorator(f):
-            self.save_function_spec (f)            
+            self.save_function_spec (f)    
+            self.set_auth_flag (f, ('testpass', testfunc.__name__))
             @wraps(f)
             def wrapper (was, *args, **kwargs):
                 response = testfunc (was)
@@ -179,14 +181,26 @@ class Decorators:
             else:
                 raise ValueError ("Unknown operator: {}".format (op))
 
-    def test_params (self, scope, required = None, ints = None, floats = None, emails = None, uuids = None, **kargs):
+    def get_parameter_requirements (self, func_id):
+        return self._parameter_caches.get (func_id, {})        
+
+    def test_params (self, scope, required = None, ints = None, floats = None, uuids = None, emails = None, **kargs):
         def decorator(f):
             self.save_function_spec (f)
+            func_id = self.get_func_id (f)
+            if func_id not in self._parameter_caches:
+                self._parameter_caches [func_id] = {}
+            self._parameter_caches [func_id][scope] = (required, ints, floats, uuids)
             @wraps(f)
             def wrapper (was, *args, **kwargs):
+                if scope in ("FORM", "JSON"):
+                    if was.request.method not in {"POST", "PUT", "PATCH"}:
+                        return f (was, *args, **kwargs)
+                    if scope == "JSON" and not was.request.get_header ("content-type", '').startswith ("application/json"):
+                        return f (was, *args, **kwargs)
                 more_info = self._validate_param (getattr (was.request, scope), required, ints, floats, emails, uuids, **kargs)
                 if more_info:
-                    return was.response.adaptive_error ("400 Bad Request", 'missing or bad parameter'.format (scope), 40050, more_info) 
+                    return was.response.adaptive_error ("400 Bad Request", 'missing or bad parameterin {}'.format (scope), 40050, more_info) 
                 return f (was, *args, **kwargs)
             return wrapper
         return decorator
