@@ -17,15 +17,18 @@ class MixIn (
     ):    
     use_reloader = False
     debug = False
-    auto_mount = True    
-    
+    auto_mount = True
+    expose_spec = False   
+    enable_namespace = False
+
     securekey = None
     session_timeout = None
 
     access_control_allow_origin = None
     access_control_max_age = 0
     authenticate = None    
-    
+    glock = threading.RLock ()
+
     def __init__ (self):
         decorators.Decorators.__init__ (self)
         router.Router.__init__ (self)
@@ -42,6 +45,7 @@ class MixIn (
         
         self.mount_p = "/"
         self.path_suffix_len = 0
+        self.service_roots = []
         self.mount_params = {}
         self.reloadables = {}
         self.reloadable_objects = {}
@@ -128,9 +132,7 @@ class MixIn (
         self.use_reloader = use_reloader
     
     # services management ----------------------------------------------
-    PACKAGE_DIRS = ["services", "decorative"]
-    CONTRIB_DIR = os.path.join (os.path.dirname (atila.__spec__.origin), 'contrib', 'services')
-                
+    PACKAGE_DIRS = ["services", "extends", "decorative"]
     def add_package (self, *names):
         for name in names:
             self.PACKAGE_DIRS.append (name)
@@ -150,7 +152,15 @@ class MixIn (
             params = self.mount_params.get (module, {})
             if params.get ("debug_only") and not self.debug:
                 return
-            
+            params ["module_name"] = module.__name__
+            if self.enable_namespace and "ns" not in params:
+                try:
+                    _, ns = module.__name__.split (".", 1)
+                    if _ not in ("services", "extends"):
+                        ns = module.__name__
+                except ValueError:
+                    ns = module.__name__
+                params ["ns"] = ns
             setattr (module, "__mntopt__", params)
             # for app initialzing and reloading
             self._mount_option = params
@@ -163,10 +173,11 @@ class MixIn (
         try:
             self.reloadables [module] = self.get_file_info (module)
         except FileNotFoundError:
-            del self.reloadables [module]
-            return        
+            del self.reloadables [module]  
+            return
+
         # find recursively
-        self.find_mountables (module)
+        self.find_mountables (module)        
 
     def _reload_objects (self, origin):
         if origin not in self.reloadable_objects:
@@ -205,9 +216,12 @@ class MixIn (
             
             if hasattr (v, "__module__"):
                 maybe_object = attr                
-                v = sys.modules [v.__module__]
-                if v == module:
-                    continue
+                try:
+                    v = sys.modules [v.__module__]
+                    if v == module:
+                        continue
+                except KeyError:                    
+                    continue        
 
             if type (v) is not ModuleType:
                 continue
@@ -230,14 +244,16 @@ class MixIn (
                     break
         
             if mountable:
-                self._mount (v)                
+                self._mount (v)
                 maybe_object and self._set_reloadable_object (maybe_object, v, module)
     
     def add_package_dir (self, path):
-        if path not in self._package_dirs:
-            self._package_dirs.append (path)        
+        for exist in self._package_dirs:
+            if exist.startswith (path):
+                return
+        self._package_dirs.append (path)                
 
-    def mount_externals (self):
+    def mount_externals (self):        
         for module in self.mount_params:            
             if module in self.reloadables:
                 continue
@@ -278,10 +294,11 @@ class MixIn (
     dettach_all = umount_all
     
     def maybe_reload (self):
-        if time.time () - self.last_reloaded < 1.0:
-            return
-        
-        self._reloading = True
+        with self.lock:  
+            if self._reloading or time.time () - self.last_reloaded < 1.0:
+                return
+            self._reloading = True
+
         for module in list (self.reloadables.keys ()):
             try:
                 fi = self.get_file_info (module)
@@ -291,18 +308,27 @@ class MixIn (
                 
             if self.reloadables [module] != fi:
                 self.log ("- reloading service, %s" % module.__name__, "info")
+                with self.glock:    
+                    for each in self.service_roots:
+                        # reinstead package root for path finder
+                        sys.modules [each.__name__] = each
+
+                    try:
+                        newmodule = reload (module)
+                    except:
+                        self.log ("- reloading failed. see exception log, %s" % module.__name__, "fatal")
+                        raise    
+                                            
                 self._current_function_specs = {}
-                if hasattr (module, "dettach"):
-                    module.dettach (self)
-                with self.lock:
-                    newmodule = reload (module)
-                    del self.reloadables [module]
-                    self._mount (newmodule)                    
-                    self._reload_objects (newmodule)
-                    
-        self.load_jinja_filters ()
-        self.last_reloaded = time.time ()        
-        self._reloading = False
+                self.umount (module)
+                del self.reloadables [module]
+                self._mount (newmodule)                    
+                self._reload_objects (newmodule)
+                self.load_jinja_filters ()       
+        
+        with self.lock:  
+            self.last_reloaded = time.time ()
+            self._reloading = False
     
     # logger ----------------------------------------------------------
     def set_logger (self, logger):
@@ -325,7 +351,7 @@ class MixIn (
     }
     def life_cycle (self, phase, obj):
         if phase in ("umounted", "before_reload"):
-            self.dettach_all ()
+            self.umount_all ()
         index = self.PHASES.get (phase)
         func = self._binds_server [index]
         if not func:
