@@ -29,14 +29,17 @@ class Router:
     
     def add_route (self, rule, func, **options):
         if rule and rule [0] != "/":
-            raise AssertionError ("Url rule should be starts with '/', {} {}".format (rule))
+            raise AssertionError ("Url rule should be starts with '/', {} for {}".format (rule, self.get_func_id (func)))
             
         func_id = self.get_func_id (func)
-        is_alter_routing = id (func) in self._function_names
+        options ["func_id"] = func_id
+        options ["mntopt"] = self._mount_option
+        options ["route"] = rule
         
-        if not is_alter_routing:
+        is_alter_routing = id (func) in self._function_names
+        if not is_alter_routing:            
             if not self._started and not self._reloading and func_id in self._function_names and "argspec" not in options:
-                self.log ("def {} is already defined. use another name or mount (ns = 'myns')".format (func_id), "warn")
+                self.log ("def {} is already defined. use another resource name, mount (ns = 'myns') or app.enable_namespace=True".format (func_id), "warn")
             
             if func_id in self._function_names and "argspec" not in options:
                 # reloading, remove old func
