@@ -44,6 +44,8 @@ class TemplateEngine:
             )
 
     def load_jinja_filters (self):    
+        if not self._jinja2_filters:
+            return
         for k, v in self._jinja2_filters.items ():
             self.jinja_env.filters [k] = v
         self._jinja2_filters = {}
