@@ -8,7 +8,7 @@ import random
 from rs4 import jwt
 from aquests.protocols.http import http_date
 from skitai import wsgiappservice 
-from skitai.wastuff import tasks
+from skitai.corequest import tasks
 import skitai
 from skitai.exceptions import HTTPError
 
@@ -31,7 +31,7 @@ class WAS (wsgiappservice.WAS):
         # override with resource default args
         if not isinstance (thing, str) or thing.startswith ("/") or thing.find (":") == -1:
             return self.app.urlfor (thing, *args, **karg)        
-        return self.apps.urlfor (thing, *args, **karg)    
+        return self.apps.build_url (thing, *args, **karg)
     ab = urlfor
     
     def partial (self, thing, **karg):
@@ -204,13 +204,12 @@ class WAS (wsgiappservice.WAS):
     def Fault (self, *args, **karg):
         return self.response.Fault (*args, **karg)
     
+    @property
+    def Error (self):
+        return HTTPError
+        
     def File (self, *args, **karg):
         return self.response.File (*args, **karg)
-        
-    def Futures (self, reqs, timeout = 10):
-        self.response.set_timeout (timeout)
-        return tasks.Futures (self._clone (True), reqs, timeout)
-    futures = Futures
     
     def proxypass (self, alias, path, timeout = 3):
         def respond (was, rss):
@@ -224,8 +223,4 @@ class WAS (wsgiappservice.WAS):
         headers = dict ([(k, v) for k, v in self.request.headers.items ()])
         headers ["Accept"] = "*/*"
         return self.Futures ([self.get ("{}/{}".format (alias, path), headers = headers)], timeout = timeout).then (respond)
-    
-    @property
-    def Error (self):
-        return HTTPError
-    
+    
\ No newline at end of file
