@@ -7,6 +7,7 @@ from aquests.athreads import trigger
 from rs4.attrdict import AttrDict
 from aquests.protocols.http import respcodes
 from skitai.exceptions import HTTPError
+from skitai.wastuff.api import API
 import warnings
 
 def traceback ():	
@@ -144,7 +145,7 @@ class Executor:
 	def parse_kargs (self, kargs):
 		self.was.request.PARAMS = kargs.copy ()
 		query = self.env.get ("QUERY_STRING")
-		data = self.was.request.dict ()
+		data = self.was.request.dict ()		
 		
 		allkarg = AttrDict ()		
 		self.merge_args (allkarg, kargs)
@@ -230,7 +231,7 @@ class Executor:
 
 		if thing:
 			request.salt = current_app.salt			
-			request.routed = current_app.get_routed (thing)
+			request.routed = current_app.get_routed (thing)			
 			request.routable = options
 		
 		current_app.maintern ()	
@@ -254,13 +255,17 @@ class Executor:
 			raise
 		except HTTPError as e:
 			self.rollback ()
-			content = request.response.with_explain (e.status, e.explain, e.errno)
+			content = [request.response.with_explain (e.status, e.explain, e.errno)]
 		except:
 			self.was.traceback ()
 			self.rollback ()			
-			content = request.response ("502 Bad Gateway", exc_info = self.was.app.debug and sys.exc_info () or None)
+			content = [request.response ("502 Bad Gateway", exc_info = self.was.app.debug and sys.exc_info () or None)]
 		else:
-			self.commit ()			
+			self.commit ()
+		
+		if current_app.debug and current_app.expose_spec and request.channel.addr [0].startswith ("127.0.0.") and isinstance (content [0], API):
+			content [0].set_spec (current_app)
+
 		# clean was
 		del self.was.subapp
 		current_app.emit ("request:finished")		
